@@ -28,9 +28,9 @@
 
 #ifdef DEBUG_DECODER
 #  include <stdio.h>
-#  define DEBUG_PRINT printf
+#  define DEBUG_PRINT(...) { printf(__VA_ARGS__); }
 #else
-#  define DEBUG_PRINT
+#  define DEBUG_PRINT(...) {}
 #endif
 
 #ifdef UNIT_TESTING
@@ -97,8 +97,8 @@ double TheengsDecoder::value_from_hex_string(const char* data_str,
 
   double value = 0;
   if (!isFloat) {
-    value = strtol(data.c_str(), NULL, 16);
-    DEBUG_PRINT("extracted value from %s = 0x%08lx\n", data.c_str(), (long)value);
+    value = strtoll(data.c_str(), NULL, 16);
+    DEBUG_PRINT("extracted value from %s = %lld\n", data.c_str(), (long long)value);
   } else {
     long longV = strtol(data.c_str(), NULL, 16);
     float floatV = *((float *) &longV);
@@ -276,7 +276,7 @@ bool TheengsDecoder::checkDeviceMatch(const JsonArray& condition,
           i++;
         }
 
-        DEBUG_PRINT("comparing value: %s to %s at index %lu\n",
+        DEBUG_PRINT("comparing value: %s to %s at index %zu\n",
                     &cmp_str[cond_index],
                     condition[i].as<const char*>(),
                     cond_index);
@@ -511,43 +511,53 @@ int TheengsDecoder::decodeBLEJson(JsonObject& jsondata) {
                       break;
                   }
                 } else {
-                  switch (*post_proc[i].as<const char*>()) {
-                    case '/':
-                      temp_val /= post_proc[i + 1].as<double>();
-                      break;
-                    case '*':
-                      temp_val *= post_proc[i + 1].as<double>();
-                      break;
-                    case '-':
-                      temp_val -= post_proc[i + 1].as<double>();
-                      break;
-                    case '+':
-                      temp_val += post_proc[i + 1].as<double>();
-                      break;
-                    case '%': {
-                      long val = (long)temp_val;
-                      temp_val = val % post_proc[i + 1].as<long>();
-                      break;
+                  if (strlen(post_proc[i].as<const char*>())== 1) {
+                    switch (*post_proc[i].as<const char*>()) {
+                      case '/':
+                        temp_val /= post_proc[i + 1].as<double>();
+                        break;
+                      case '*':
+                        temp_val *= post_proc[i + 1].as<double>();
+                        break;
+                      case '-':
+                        temp_val -= post_proc[i + 1].as<double>();
+                        break;
+                      case '+':
+                        temp_val += post_proc[i + 1].as<double>();
+                        break;
+                      case '%': {
+                        long val = (long)temp_val;
+                        temp_val = val % post_proc[i + 1].as<long>();
+                        break;
+                      }
+                      case '<': {
+                        long val = (long)temp_val;
+                        temp_val = val << post_proc[i + 1].as<unsigned int>();
+                        break;
+                      }
+                      case '>': {
+                        long val = (long)temp_val;
+                        temp_val = val >> post_proc[i + 1].as<unsigned int>();
+                        break;
+                      }
+                      case '!': {
+                        bool val = (bool)temp_val;
+                        temp_val = !val;
+                        break;
+                      }
+                      case '&': {
+                        long long val = (long long)temp_val;
+                        temp_val = val & post_proc[i + 1].as<unsigned int>();
+                        break;
+                      }
                     }
-                    case '<': {
-                      long val = (long)temp_val;
-                      temp_val = val << post_proc[i + 1].as<unsigned int>();
-                      break;
+                  } else if (strncmp(post_proc[i].as<const char*>(), "max", 3) == 0) {
+                    if (temp_val > post_proc[i + 1].as<double>()) {
+                      temp_val = post_proc[i + 1].as<double>();
                     }
-                    case '>': {
-                      long val = (long)temp_val;
-                      temp_val = val >> post_proc[i + 1].as<unsigned int>();
-                      break;
-                    }
-                    case '!': {
-                      bool val = (bool)temp_val;
-                      temp_val = !val;
-                      break;
-                    }
-                    case '&': {
-                      long val = (long)temp_val;
-                      temp_val = val & post_proc[i + 1].as<unsigned int>();
-                      break;
+                  } else if (strncmp(post_proc[i].as<const char*>(), "min", 3) == 0) {
+                    if (temp_val < post_proc[i + 1].as<double>()) {
+                      temp_val = post_proc[i + 1].as<double>();
                     }
                   }
                 }
@@ -559,15 +569,15 @@ int TheengsDecoder::decodeBLEJson(JsonObject& jsondata) {
                 */
             std::string _key = sanitizeJsonKey(kv.key().c_str());
 
-            /* calculation values extracted from data are not added to the deocded outupt
-                * instead we store them teporarily to use with the next data properties.
+            /* calculation values extracted from data are not added to the decoded output
+                * instead we store them temporarily to use with the next data properties.
                 */
             if (_key == ".cal") {
               cal_val = temp_val;
               continue;
             }
 
-            /* Cast to a differnt value type if specified */
+            /* Cast to a different value type if specified */
             if (prop.containsKey("is_bool")) {
               jsondata[_key] = (bool)temp_val;
             } else {
@@ -582,6 +592,14 @@ int TheengsDecoder::decodeBLEJson(JsonObject& jsondata) {
               _key[4] = 'c';
             }
 
+            /* If the property is with suffix _cm, make sure to convert and add length in inches */
+            if (_key.find("_cm", _key.length() - 3, 3) != std::string::npos) {
+              double tc = jsondata[_key];
+              _key.replace(_key.length() - 3, 3, "_in");
+              jsondata[_key] = tc / 2.54;
+              _key.replace(_key.length() - 3, 3, "_cm");
+            }
+
             success = i_main;
             DEBUG_PRINT("found value = %s : %.2f\n", _key.c_str(), jsondata[_key].as<double>());
           } else if (strstr((const char*)decoder[0], "static_value") != nullptr) {
